lrm.las <- function(X, y, group, lambda, beta, mu,
                    eps.outer, eps.inner, max.it, optim.func){
  
  ## INITIALIZATION
  p  <- ncol(X)
  active <- abs(beta) > 0
  iter.optim <- c()
  dL.min  <- dL.lasso.binomial(c(mu,beta),X,y,lambda)
  gap.old <- max(abs(dL.min))
  ## Check if optimality is reached at starting point
  if (gap.old < eps.outer*p) {
    return(list(beta=beta,mu=mu,iter.optim=iter.optim,gap=gap.old,status="converged"))
  } else {
    ## UPDATE THE ACTIVE SET / ACTIVE GROUP
    if (is.null(mu)) {
      new <- which.max(abs(dL.min))
    } else {
      new <- which.max(abs(dL.min[-1]))
    }
    if (all(active[new])) {
      eps.optim  <- eps.outer
    } else {
      active[new] <- TRUE
      eps.optim  <- eps.inner
    }
  }

  while (1) {
    
    ## (1) OPTIMIZATION OVER A
    ## _____________________________________________________________
    res <- optim.func(as.matrix(c(mu,beta[active])), X[,active], y,
                      lambda[active], eps.optim)
    iter.optim <- c(iter.optim,res$i)
    if (is.null(mu)) {
      beta[active] <- res$xk
      dL.min[active] <- dL.lasso.binomial(beta[active],
                                          X[,active],y,lambda[active])
    } else {
      mu <- res$xk[1]
      beta[active] <- res$xk[-1]
      dL.min[c(TRUE,active)] <- dL.lasso.binomial(c(mu,beta[active]),
                                                  X[,active], y, lambda[active])
    }
    
    ## (2) TEST LAGRAGIAN VIOLATION AND GROUP DELETION IF APPLICABLE
    ## _____________________________________________________________
    go.zero <- abs(beta) < eps.optim & active
    if (sum(go.zero) > 0) {
      if (is.null(mu)) {
        zeroed <- go.zero & abs(dL.min) < eps.optim
        dL.min[zeroed] <- 0
      } else {
        zeroed <- go.zero & abs(dL.min[-1]) < eps.optim
        dL.min[c(FALSE,zeroed)] <- 0
      }
      active[zeroed] <- FALSE
      beta[zeroed]   <- 0      
    }

    ## (3) OPTIMALITY TESTING AND GROUP ACTIVATION IF APPLICABLE
    ## _____________________________________________________________
    gap <- max(abs(dL.min))    
    if (length(iter.optim) == max.it) {
      return(list(beta=beta,mu=mu,iter.optim=iter.optim,
                  status="max # of iterates reached",gap=gap))
    }
    if (gap < eps.outer*p) {
      return(list(beta=beta,mu=mu,iter.optim=iter.optim,gap=gap,status="converged"))
    } else {
      ## UPDATE THE ACTIVE SET / ACTIVE GROUP
      if (is.null(mu)) {
        new <- which.max(abs(dL.min))
      } else {
        new <- which.max(abs(dL.min)[-1])
      }
      if (all(active[new])) {
        eps.optim <- eps.outer
      } else {
        active[new] <- TRUE
        eps.optim <- eps.inner
      }
    }
  }
}

L.lasso.binomial <- function(beta,X,y,lambda) {

  ## Managing intercept
  if (length(beta) == length(lambda)) {
      eta <- crossprod(t(X), beta)
  } else {
    eta <- cbind(1,X) %*% beta
    lambda <- c(0,lambda)
  }
  
  L  <- - sum( y * eta - log(1 + exp(eta)) ) + sum(lambda * abs(beta))

  return(L)
}

dL.lasso.binomial <- function(beta,X,y,lambda) {

  zero <- abs(beta) == 0
  dL <- rep(0,length(beta))

  ## Managing intercept
  if (length(beta) == length(lambda)) {
    eta <- crossprod(t(X),beta)
    nabla.f <- - crossprod(X,y-1/(1+exp(-eta)))
  } else {
    eta <- cbind(1,X) %*% beta
    nabla.f <- - crossprod(cbind(1,X),y-1/(1+exp(-eta)) )
    lambda <- c(0,lambda)
  }
  n1.nabla.f <- abs(nabla.f)
  
  ## terms whose norm1 is different from zero
  dL[!zero] <- nabla.f[!zero] + lambda[!zero] * sign(beta)[!zero]
  
  ## terms whose norm2 is zero
  to.zero    <- n1.nabla.f[zero] <= lambda[zero]
  
  ## which ones can be null ?
  dL[zero][to.zero] <- 0
  dL[zero][!to.zero] <- nabla.f[zero][!to.zero] * (1 - lambda[zero][!to.zero]/n1.nabla.f[zero][!to.zero])

  return(dL)
}  

lrm.grp <- function(X, y, group, lambda, beta, mu,
                    eps.outer, eps.inner, max.it, optim.func){
  
  ## INITIALIZATION
  p  <- ncol(X)
  pk <- tabulate(factor(group))
  active <- group.norm.rep(beta, pk) > 0
  iter.optim <- c()
  ## Check if optimality is reached at starting point
  dL.min <- dL.group.binomial(c(mu,beta),X,y,lambda,pk)
  if (is.null(mu)) {
    n2.dL   <- group.norm(dL.min,pk)
  } else {
    n2.dL   <- group.norm(dL.min[-1],pk)
  }
  gap.old <- max(c(abs(dL.min[1]),n2.dL)) 
  if (gap.old < eps.outer*p) {
    return(list(beta=beta,mu=mu,iter.optim=iter.optim,gap=gap.old,status="converged"))
  } else {
    ## UPDATE THE ACTIVE SET / ACTIVE GROUP
    new <- which.max(n2.dL)
    if (all(active[group == new])) {
      eps.optim  <- eps.outer
    } else {
      active[group == new] <- TRUE
      eps.optim  <- eps.inner
    }
  }

  while (1) {
    
    ## (1) OPTIMIZATION OVER A
    ## _____________________________________________________________
    res <- optim.func(as.matrix(c(mu,beta[active])), X[,active], y,
                      lambda[active], tabulate(factor(group[active])), eps.optim)
    iter.optim <- c(iter.optim,res$i)
    if (is.null(mu)) {
      beta[active] <- res$xk
      dL.min[active] <- dL.group.binomial(beta[active],X[,active],y,
                                          lambda[active],
                                          tabulate(factor(group[active])))
    } else {
      mu <- res$xk[1]
      beta[active] <- res$xk[-1]
      dL.min[c(TRUE,active)] <- dL.group.binomial(c(mu,beta[active]),X[,active],y,
                                                  lambda[active],
                                                  tabulate(factor(group[active])))
    }
    
    ## (2) TEST LAGRAGIAN VIOLATION AND GROUP DELETION IF APPLICABLE
    ## _____________________________________________________________
    go.zero <- intersect(which(group.norm(beta,pk) < eps.optim),group[active])
    if (length(go.zero) > 0) {
      if (is.null(mu)) {
        n2.dL  <- group.norm.rep(dL.min,pk)
        zeroed <- group %in% go.zero & n2.dL < eps.optim
        dL.min[zeroed] <- 0
      } else {
        n2.dL  <- group.norm.rep(dL.min[-1],pk)
        zeroed <- group %in% go.zero & n2.dL < eps.optim
        dL.min[c(FALSE,zeroed)] <- 0
      }
      active[zeroed] <- FALSE
      beta[zeroed]   <- 0
    }

    ## (3) OPTIMALITY TESTING AND GROUP ACTIVATION IF APPLICABLE
    ## _____________________________________________________________
    if (is.null(mu)) {
      n2.dL   <- group.norm(dL.min,pk)
    } else {
      n2.dL   <- group.norm(dL.min[-1],pk)
    }
    gap <- max(c(abs(dL.min[1]),n2.dL)) 
    if (length(iter.optim) == max.it) {
      return(list(beta=beta,mu=mu,iter.optim=iter.optim,
                  status="max # of iterates reached",gap=gap))
    }    
    if (gap < eps.outer*p) {
      return(list(beta=beta,mu=mu,iter.optim=iter.optim,gap=gap,status="converged"))
    } else {
      ## UPDATE THE ACTIVE SET / ACTIVE GROUP
      new <- which.max(n2.dL)
      if (all(active[group == new])) {
        eps.optim <- eps.outer
      } else {
        active[group == new] <- TRUE
        eps.optim <- eps.inner
      }
    }
  }
}

L.group.binomial <- function(beta,X,y,lambda,pk) {

  ## Managing intercept
  if (length(beta) == length(lambda)) {
    eta <- crossprod(t(X), beta)
    n2 <- group.norm.rep(beta,pk) / rep.int(pk,pk)
  } else {
    eta <- crossprod(t(cbind(1,X)), beta)
    n2 <- group.norm.rep(beta[-1],pk) / rep.int(pk,pk)
  }
  
  L  <- - sum( y * eta - log(1 + exp(eta)) ) + sum(lambda *n2)

  return(L)
}

dL.group.binomial <- function(beta,X,y,lambda,pk) {

  ## Managing intercept
  if (length(beta) == length(lambda)) {
    eta <- crossprod(t(X),beta)
    nabla.f <- - crossprod(X, y-1/(1+exp(-eta)) )
    n2.nabla.f <- group.norm.rep(nabla.f,pk)
    n2 <- group.norm.rep(beta,pk)
  } else {
    eta <- cbind(1,X) %*% beta
    nabla.f <- - crossprod(cbind(1,X), y-1/(1+exp(-eta)) )
    n2.nabla.f <- c(abs(nabla.f[1]),group.norm.rep(nabla.f[-1],pk))
    lambda <- c(0,lambda)
    n2 <- c(abs(beta[1]),group.norm.rep(beta[-1],pk))
  }

  zero <- n2 == 0
  ## terms whose norm2 is different from zero
  dL <- rep(0,length(beta))
  dL[!zero] <- nabla.f[!zero] + lambda[!zero] * beta[!zero] / n2[!zero]
  
  ## terms whose norm2 is zero
  to.zero    <- n2.nabla.f[zero] <= lambda[zero]
  
  ## which ones can be null ?
  dL[zero][to.zero] <- 0
  dL[zero][!to.zero] <- nabla.f[zero][!to.zero] * (1 - lambda[zero][!to.zero]/n2.nabla.f[zero][!to.zero])

  return(dL)
}  

lrm.coo <- function(X, y, group, lambda, beta, mu,
                    eps.outer, eps.inner, max.it, optim.func){
  
  ## INITIALIZATION
  max.n2.dL.old <- Inf
  p  <- ncol(X)
  pk <- tabulate(factor(group))
  active.pos <- group.norm.rep(pmax.int(0, beta),pk) > 0
  active.neg <- group.norm.rep(pmax.int(0,-beta),pk) > 0
  iter.optim <- c()
  ## Check if optimality is reached at the starting point
  dL.min <- dL.coop.binomial(c(mu,beta),X,y,lambda,pk)
  if (is.null(mu)) {
    n2.dL.pos <- group.norm(pmax.int(0, dL.min),pk)
    n2.dL.neg <- group.norm(pmax.int(0,-dL.min),pk)
  } else {
    n2.dL.pos <- group.norm(pmax.int(0, dL.min[-1]),pk)
    n2.dL.neg <- group.norm(pmax.int(0,-dL.min[-1]),pk)
  }
  gap.old <- max(c(abs(dL.min[1]),pmax(n2.dL.pos,n2.dL.neg))) 
  if (gap.old < eps.outer*p) {
    return(list(beta=beta,mu=mu,iter.optim=iter.optim,gap=gap.old,status="converged"))
  } else {
    ## UPDATE THE ACTIVE SET / ACTIVE GROUP
    new <- which.max(pmax(n2.dL.pos,n2.dL.neg))
    if (all(active.neg[group == new]) | all(active.pos[group == new])) {
        eps.optim <- eps.outer
    } else {
        eps.optim <- eps.inner
    }
    if (n2.dL.pos[new] > n2.dL.neg[new]) {
      active.neg[group == new] <- TRUE
    } else {
      active.pos[group == new] <- TRUE
    }
  }
  
  while (1) {

    ## (1) OPTIMIZATION OVER A
    ## _____________________________________________________________
    lower.bound <- rep(-Inf,p)
    upper.bound <- rep(+Inf,p)
    lower.bound[active.pos & !active.neg] <- 0
    upper.bound[active.neg & !active.pos] <- 0
    active <- active.pos | active.neg
    res <- optim.func(as.matrix(c(mu,beta[active])), X[,active], y,
                      lambda[active],tabulate(factor(group[active])),eps.optim,
                      lower = lower.bound[active], upper = upper.bound[active])
    iter.optim <- c(iter.optim,res$i)
    if (is.null(mu)) {
      beta[active] <- res$xk
      dL.min[active] <- dL.coop.binomial(c(mu,beta[active]),X[,active],y,
                                         lambda[active],
                                         tabulate(factor(group[active])))
    } else {
      mu <- res$xk[1]
      beta[active] <- res$xk[-1]
      dL.min[c(TRUE,active)] <- dL.coop.binomial(c(mu,beta[active]),X[,active],y,
                                                 lambda[active],
                                                 tabulate(factor(group[active])))
    }
    
    ## (2) TEST LAGRAGIAN VIOLATION AND GROUP DELETION IF APPLICABLE
    ## _____________________________________________________________
    n2.beta.pos <- group.norm(pmax.int(0,beta),pk)
    go.zero.pos <- intersect(which(n2.beta.pos < eps.optim),group[active.pos])
    if (length(go.zero.pos) > 0) {
      if (is.null(mu)) {
        n2.dL.neg  <- group.norm.rep(pmax.int(0,-dL.min),pk)
        zeroed <- group %in% go.zero.pos & n2.dL.neg < eps.optim & beta > 0
        dL.min[zeroed] <- 0
      } else {
        n2.dL.neg  <- group.norm.rep(pmax.int(0,-dL.min[-1]),pk)
        zeroed <- group %in% go.zero.pos & n2.dL.neg < eps.optim & beta > 0
        dL.min[c(FALSE,zeroed)] <- 0
      }
      active.pos[zeroed] <- FALSE
      beta[zeroed]       <- 0
    }

    n2.beta.neg <- group.norm(pmax.int(0,-beta),pk)
    go.zero.neg <- intersect(which(n2.beta.neg < eps.optim),group[active.neg])
    if (length(go.zero.neg) > 0) {
      if (is.null(mu)) {
        n2.dL.pos  <- group.norm.rep(pmax.int(0,dL.min),pk)
        zeroed <- group %in% go.zero.neg & n2.dL.pos < eps.optim & beta < 0
        dL.min[zeroed] <- 0
      } else {
        n2.dL.pos  <- group.norm.rep(pmax.int(0,dL.min[-1]),pk)
        zeroed <- group %in% go.zero.neg & n2.dL.pos < eps.optim & beta < 0
        dL.min[c(FALSE,zeroed)] <- 0
      }
      active.neg[zeroed] <- FALSE
      beta[zeroed]       <- 0
    }
    
    ## (3) OPTIMALITY TESTING AND GROUP ACTIVATION IF APPLICABLE
    ## _____________________________________________________________
    ## Get the entry for which the constraint is the most violated
    if (is.null(mu)) {
      n2.dL.pos <- group.norm(pmax.int(0, dL.min),pk)
      n2.dL.neg <- group.norm(pmax.int(0,-dL.min),pk)
    } else {
      n2.dL.pos <- group.norm(pmax.int(0, dL.min[-1]),pk)
      n2.dL.neg <- group.norm(pmax.int(0,-dL.min[-1]),pk)
    }
    pmax.n2.dL <- pmax.int(n2.dL.pos,n2.dL.neg)
    gap <- max(c(abs(dL.min[1]),pmax.n2.dL))
    if (length(iter.optim) == max.it) {
      return(list(beta=beta,mu=mu,iter.optim=iter.optim,status="max # of iterates reached",
                  gap=gap))
    }
    if (abs(gap - gap.old) < eps.outer) {
      return(list(beta=beta,mu=mu,iter.optim=iter.optim,status="no further improvement...",
                  gap=gap))
    }
    
    if (gap < eps.outer*p) {
      return(list(beta=beta,mu=mu,iter.optim=iter.optim,gap=gap,status="converged"))
    } else {
      gap.old <- gap
      ## UPDATE THE ACTIVE SET / ACTIVE GROUP
      new <- which.max(pmax.n2.dL)
      if (all(active.neg[group == new]) | all(active.pos[group == new])) {
        eps.optim <- eps.outer
      } else {
        eps.optim <- eps.inner
      }
      if (n2.dL.pos[new] > n2.dL.neg[new]) {
        active.neg[group == new] <- TRUE
      } else {
        active.pos[group == new] <- TRUE
      }
    }
  }
}

L.coop.binomial <- function(beta,X,y,lambda,pk) {

  ## Managing intercept
  if (length(beta) == length(lambda)) {
    eta <- crossprod(t(X), beta)
    a.pos <- group.norm.rep(pmax.int(0, beta),pk)/rep.int(pk,pk)
    a.neg <- group.norm.rep(pmax.int(0,-beta),pk)/rep.int(pk,pk)
  } else {
    eta <- crossprod(t(cbind(1,X)), beta)
    a.pos <- group.norm.rep(pmax.int(0, beta[-1]),pk)/rep.int(pk,pk)
    a.neg <- group.norm.rep(pmax.int(0,-beta[-1]),pk)/rep.int(pk,pk)
  }
  
  L <- - sum( y * eta - log(1 + exp(eta)) ) + sum(lambda *(a.pos + a.neg))

  return(L)
}

dL.coop.binomial <- function(beta,X,y,lambda,pk) {

  ## Managing intercept
  if (length(beta) == length(lambda)) {
    inter <- FALSE
    eta <- crossprod(t(X), beta)
    nabla.f <- - crossprod(X, y-1/(1+exp(-eta)) )
    a.pos <- group.norm.rep(pmax.int(0, beta),pk)
    a.neg <- group.norm.rep(pmax.int(0,-beta),pk)
    n2.theta.pos <- group.norm.rep(pmax.int(0, nabla.f),pk)
    n2.theta.neg <- group.norm.rep(pmax.int(0,-nabla.f),pk)
  } else {
    inter <- TRUE
    eta <- crossprod(t(cbind(1,X)), beta)
    nabla.f <- - crossprod(cbind(1,X), y-1/(1+exp(-eta)) )
    lambda <- c(0,lambda)
    a.pos <- c(max(0, beta[1]),group.norm.rep(pmax.int(0, beta[-1]),pk))
    a.neg <- c(max(0,-beta[1]),group.norm.rep(pmax.int(0,-beta[-1]),pk))
    n2.theta.pos <- c(max(0, nabla.f[1]),group.norm.rep(pmax.int(0, nabla.f[-1]),pk))
    n2.theta.neg <- c(max(0,-nabla.f[1]),group.norm.rep(pmax.int(0,-nabla.f[-1]),pk))
  }
    
  ## manage cases where alpha+ > 0 or alpha- < 0
  dL <- nabla.f
  neg <- beta < 0
  pos <- beta > 0
  dL[pos] <- nabla.f[pos] + lambda[pos]*pmax.int(rep(0,sum(pos)), beta[pos]) / a.pos[pos]
  dL[neg] <- nabla.f[neg] - lambda[neg]*pmax.int(rep(0,sum(neg)),-beta[neg]) / a.neg[neg]
  
  ## manage cases where beta = 0 and (alpha- = 0 and/or alpha+ = 0)
  zero.pos  <- abs(beta) == 0 & nabla.f > 0 & a.neg == 0
  zero.neg  <- abs(beta) == 0 & nabla.f < 0 & a.pos == 0
  
  pos.to.zero  <- n2.theta.pos[zero.pos] <= lambda[zero.pos]
  neg.to.zero  <- n2.theta.neg[zero.neg] <= lambda[zero.neg]

  dL[zero.pos][!pos.to.zero] <- nabla.f[zero.pos][!pos.to.zero] * (1-lambda[zero.pos][!pos.to.zero]/n2.theta.pos[zero.pos][!pos.to.zero])
  dL[zero.neg][!neg.to.zero] <- nabla.f[zero.neg][!neg.to.zero] * (1-lambda[zero.neg][!neg.to.zero]/n2.theta.neg[zero.neg][!neg.to.zero])
  
  dL[zero.pos][pos.to.zero] <- 0
  dL[zero.neg][neg.to.zero] <- 0
  
  return(dL)
}
