is.scoop <- function(o) {
  return(class(o) == "scoop")
}

is.cvscoop <- function(o) {
   return(class(o) == "cvscoop")
}

is.intercept <- function(o) {
  ncoef <- ncol(o$coefficients)
  if (is.null(ncoef)) {ncoef <- length(o$coefficients)}
  if (length(o$group) == ncoef) {
    return(rep(FALSE,ncoef))
  } else {
    return(c(TRUE,rep(FALSE,ncoef-1)))
  }
}

print.scoop <- function(x, ...) {
  stopifnot(is.scoop(x))

  ncoef <- ncol(x$coefficients)
  if (is.null(ncoef)) {ncoef <- length(x$coefficients)}
  cat("\nCall:", deparse(x$call), "\n")
  cat("- number of coefficients:", ncoef, "\n")
  cat("- number of groups:", length(unique(x$group)), "\n")
  cat("- penalty parameter lambda:", length(x$lambda), "points from",
      format(max(x$lambda), digits = 3),"to",
      format(min(x$lambda), digits = 3))
  cat("\n")
  invisible(x)
}

plot.scoop <- function(x, xvar = "lambda", yvar="coefficients",
                         main = paste(x$penalty," path (", yvar, ")", sep=""),
                         crit = NULL, log.scale = TRUE, labels = NULL,
                         col = c(1+switch(yvar,
                           group = c(1:nlevels(as.factor(x$group))),
                           c(x$group))),
                         lty = switch(yvar,
                           group = c(1:nlevels(as.factor(x$group))),
                           c(x$group)), ...) {
    
  stopifnot(is.scoop(x))

  if (length(x$lambda) == 1) {
    stop("Not available when length(lambda) == 1")
  }

  ## removing intercept for plotting
  beta   <- x$coefficients[,!is.intercept(x)]
  if (yvar == "group") {
    beta <- t(apply(beta,1,group.norm.unordered,group=as.factor(x$group)))
  }

  xv <- switch(xvar,
               "df"       = df.scoop(x),
               "fraction" = apply(abs(beta),1,sum)/max(apply(abs(beta),1,sum)),
               ifelse(rep(log.scale,length(x$lambda)),log10(x$lambda),x$lambda))
  xlab <- switch(xvar,
                 "df"       = "degrees of freedom",
                 "fraction" = "fraction",
                 ifelse(log.scale, "lambda (log scale)", "lambda"))
  
  matplot(xv, beta, xlab = xlab, ylab = yvar, col=col, lty=lty, main=main,
          type="l", yaxt = switch(!is.null(labels), T="n", "s"), ...)
  abline(h = 0, lty = 3)
  
  if (!is.null(labels)) { 
    axis(switch(xvar,"lambda"=2,4), at = beta[which.max(rowSums(abs(beta))),],
         labels = labels, cex.axis=.65, las=2)
  }
  
  if (!is.null(crit)) {
    x.star <- xv[max(which(crit == min(crit, na.rm=TRUE)), na.rm=TRUE)]
    abline(v=x.star)
  }
}

fitted.scoop <- function(object, ...) {

  stopifnot(is.scoop(object))

  return(predict(object))
}

predict.scoop <- function (object, newx=NULL, ...)  {

  stopifnot(is.scoop(object))

  if (is.null(newx)) {newx <- object$x}
  
  if (object$family == "gaussian") {
    if (sum(is.intercept(object))) {
      intercept <- matrix(rep(t(object$coefficient[,1]), nrow(newx)),
                          nrow=nrow(newx), byrow=TRUE)
      return(intercept + newx %*% t(object$coefficients[,-1]))
    } else {
      return(newx %*% t(object$coefficients))
    }
  }

  if (object$family == "binomial") {
    if (sum(is.intercept(object))) {
      intercept <- matrix(rep(t(object$coefficient[,1]), nrow(newx)),
                          nrow=nrow(newx), byrow=TRUE)
      eta <- as.matrix(newx %*% t(object$coefficients[,-1]))
      return(sigmoid(intercept + eta))
    } else {
      return(sigmoid(newx %*% t(object$coefficients)))
    }    
  }  
}

residuals.scoop <- function(object, ...) {

  stopifnot(is.scoop(object))
  n <- length(object$lambda)
  
  return(matrix(rep(object$y, n), ncol=n) - fitted(object))
}

deviance.scoop <- function(object, ...) {

  stopifnot(is.scoop(object))
  
  if (object$family == "gaussian") {
    dev <- apply(residuals(object)^2,2,sum)
  }
  if (object$family == "binomial") {
    y.hat <- predict(object)
    y     <- matrix(rep(as.matrix(object$y),ncol(y.hat)),ncol=ncol(y.hat))
    dev <- 2*colSums((log(1-y))^(1-y) + (log(y))^y - (log(1-y.hat))^(1-y)
                     - (log(y.hat))^y)
  }
  return(dev)

}

df.scoop <- function(o) {

  stopifnot(is.scoop(o))

  if (o$family == "binomial") {
    warning("df for binomial family not implemented - returning NULL.")
    return(NULL)
  }
  
  if (sum(is.intercept(o))) {
    beta   <- coefficients(o)[,-1]
    beta.reg <- coefficients(o$reg.fit)[-1]
  } else {
    beta   <- coefficients(o)
    beta.reg <- coefficients(o$reg.fit)
  }
  group <- o$group

  if (is.null(o$reg.fit$lambda)) {
    gamma <- 0
  } else {
    gamma <- o$reg.fit$lambda
  }
  
  L <- nrow(beta)
  g <- length(table(group))

  if (o$penalty == "lasso") {
    df <- rowSums(o$coefficients != 0)  
  } else
  if (o$penalty == "group") {

    n2.reg <- group.norm.unordered(beta.reg,group)
    pg <- tapply(beta.reg != 0,group,sum)
    
    n2       <- apply(as.matrix(beta),1,group.norm.unordered,group)
    n2.scale <- t(scale(t(n2),FALSE,n2.reg/((pg-1) / (1+gamma) )))

    I <- matrix(0,ncol=L,nrow=g)
    I[n2>0] <- 1
    
    df <- colSums( I * (1 + n2.scale) )
    
  } else
  if (o$penalty == "coop") {

    n2.reg.p <- group.norm.unordered(pmax(0,beta.reg),group)
    n2.reg.m <- group.norm.unordered(pmin(0,beta.reg),group)
    pg.p     <- tapply(beta.reg > 0, group,sum)
    pg.m     <- tapply(beta.reg < 0, group,sum)

    ## Manage 0/0 = 0
    n2.reg.p[n2.reg.p == 0] <- Inf
    n2.reg.m[n2.reg.m == 0] <- Inf
    
    n2.p <- apply(matrix(pmax(0,beta),nrow=L), 1, group.norm.unordered, group)
    n2.m <- apply(matrix(pmin(0,beta),nrow=L), 1, group.norm.unordered, group)
    
    n2.scale.p  <- t(scale(t(n2.p), FALSE, n2.reg.p / ( (pg.p-1)/(1+gamma) ) ))
    n2.scale.m  <- t(scale(t(n2.m), FALSE, n2.reg.m / ( (pg.m-1)/(1+gamma) ) ))
    
    Ip  <- matrix(0,ncol=L,g)
    Im <- matrix(0,ncol=L,g)
    Ip[n2.p>0] <- 1
    Im[n2.m>0] <- 1
    df <- colSums( Ip*(1 + n2.scale.p) + Im*(1 + n2.scale.m ) )
  }

  if (sum(is.intercept(o))) {
    df <- df + 1
  }
  names(df) <- round(o$lambda,2)

  return(df)
}

selection <- function(object, ...) {
  UseMethod("selection")
}

selection.scoop <- function(object, sigma2=NULL, ...) {

  stopifnot(is.scoop(object))
  stopifnot(object$family == "gaussian" |
            (object$family == "binomial" & object$penalty == "lasso"))

  Beta   <- coefficients(object)
  lambda <- object$lambda
  n  <- nrow(object$x)
  df <- df.scoop(object)
  L  <- length(lambda)
  p  <- ncol(Beta)
  if (sum(is.intercept(object))) {
    p <- p-1
  }
  
  BIC <- beta.BIC <- lambda.BIC <- df.BIC <- NA
  AIC <- beta.AIC <- lambda.AIC <- df.AIC <- NA

  RSS <- deviance(object) 
  
  if (length(RSS) < p & (is.null(sigma2))) {
    sigma2 <- NA
    stop("Cannot compute OLS nor estimate sigma2 in the high dimensional context.")
  }
    
  if (is.null(sigma2)) {
    sigma2 <- deviance(object$reg.fit)/df.residual(object$reg.fit)
  }
  
  ## BIC choice
  BIC    <- RSS / sigma2  + log(n) * df
  choice <- which.min(BIC)
  beta.BIC   <- Beta[choice,]
  lambda.BIC <- lambda[choice]
  
  ## AIC choice
  AIC    <- RSS / sigma2 + 2*df
  choice <- which.min(AIC)
  beta.AIC   <- Beta[choice,]
  lambda.AIC <- lambda[choice]

  return(list(BIC=BIC, beta.BIC=beta.BIC, lambda.BIC=lambda.BIC, 
              AIC=AIC, beta.AIC=beta.AIC, lambda.AIC=lambda.AIC, 
              sigma2 = sigma2, df = df, non.zeros = rowSums(object$coef != 0)))
  
}

crossval <- function(object, ...) {
  UseMethod("crossval")
}

crossval.scoop <- function (object,
                            K          = 10,
                            error      = "deviance",
                            threshold  = 0.5,
                            folds      = NULL,
                            verbose    = TRUE, ...) {
  
  stopifnot(is.scoop(object))

  ## =====================================================================
  ## INITIALIZATION
  ## extract all variables needed from object scoop
  x         <- object$x
  y         <- object$y
  group     <- object$group
  family    <- object$family
  penalty   <- object$penalty
  intercept <- sum(is.intercept(object)) > 0
  lambda    <- object$lambda

  ## some more work to extract eps and normalize from the call to scoop
  call.eps <- match(c("eps"), names(object$call), 0L)
  if (call.eps == 0) {
    eps <- 1e-5
  } else {
    if(is.numeric(object$call[call.eps][[1]])) {
      eps <- object$call[call.eps][[1]]
    } else {
      eps <- 1e-5
    }
  }
  call.normalize <- match(c("normalize"), names(object$call), 0L)
  if (call.normalize == 0) {
    normalize <- TRUE
  } else {
    if (is.logical(object$call[call.normalize][[1]])) {
      normalize <- object$call[call.normalize][[1]]
    } else {
      normalize <- TRUE
    }
  }
  
  ## =====================================================================
  ## CROSS-VALIDATION WORK

  ## if folds are specified by the user
  if (is.null(folds)) {
    folds <- split(sample(1:length(y)), rep(1:K, length = length(y)))
  } else {
    K <- length(folds)
  }

  ## now loop on the fold and cross-validate
  if(verbose) {cat("omitting bloc")}
  err <- matrix(0, length(lambda), K)
  for (i in seq(K)) {
    if(verbose) {cat("",i)}

    omit <- folds[[i]]
    fit <- scoop(x[-omit,], y[-omit], group, penalty = penalty, family=family,
                   intercept = intercept, normalize = normalize, lambda =lambda,
                   eps = eps)
    
    if (error == "class" & family == "binomial") {
      y.hat <- predict(fit,x[omit,])     
      y.hat[y.hat  > threshold] <- 1
      y.hat[y.hat <= threshold] <- 0
      err[,i] <- apply( (y[omit] - y.hat) ^2,2,mean)
    } else {
      err[,i] <- err.deviance(fit, x[omit,], y[omit]) / length(omit)
    }
  }

  ## average over folds
  cv.mean     <- apply(err, 1, mean)
  cv.error    <- sqrt(apply(err, 1, var)/K)

  lambda.min <- max(lambda[cv.mean <= min(cv.mean)])
  lambda.1se <- max(lambda[cv.mean <  (cv.mean + cv.error)[cv.mean <= min(cv.mean)]])
  beta.cv <- object$coefficients[min(which(cv.mean <= min(cv.mean)) ),]
  
  out <- list(lambda=lambda, lambda.min=lambda.min, lambda.1se=lambda.1se,
              cv.mean=cv.mean, cv.error=cv.error, folds=folds, beta.cv=beta.cv)
  class(out) <- "cvscoop"
  return(out)
}

err.deviance <- function(o, x, y) {

  stopifnot(is.scoop(o))
  
  if (o$family == "gaussian") {
    dev <- apply((y-predict(o,x))^2,2,sum)
  }
  if (o$family == "binomial") {
    y.hat <- predict(o, x)
    y     <- matrix(rep(as.matrix(y),ncol(y.hat)),ncol=ncol(y.hat))    
    dev <- 2*colSums((log(1-y))^(1-y) + (log(y))^y - (log(1-y.hat))^(1-y)
                     - (log(y.hat))^y)
  }
  return(dev)

}

plot.cvscoop <- function(x, xlab = NULL, log.scale = TRUE,
                         ylab = "Mean CV error",  ...) {

  stopifnot(is.cvscoop(x))

  if(log.scale){
    xv <- log10(x$lambda)
    l.min <- log10(x$lambda.min)
    l.1se <- log10(x$lambda.1se)
  } else {
    xv <- x$lambda
    l.min <- x$lambda.min
    l.1se <- x$lambda.min
  }
  
  if (is.null(xlab)) {
    xlab <- ifelse(log.scale, "lambda (log scale)", "lambda")
  }
  
  upper <- x$cv.mean + x$cv.error
  lower <- x$cv.mean - x$cv.error
  
  plot(xv, x$cv.mean, type="b", xlab=xlab, ylab=ylab,
       ylim=range(upper,lower, na.rm=TRUE), col="red", ...)
  barw <- range(xv) / length(xv)
  segments(xv, upper, xv, lower, col="gray")
  segments(xv - barw, upper, xv + barw, upper, col="gray")
  segments(xv - barw, lower, xv + barw, lower, col="gray")
  range(upper, lower)
  abline(v=l.min)
  axis(3, at = l.min, labels = "cross val min", cex.axis=0.75)
  abline(v=l.1se)
  axis(3, at = l.1se, labels = "cross val 1se", cex.axis=0.75)
}

reg.fitting <- function(x, y, intercept, family) {
  
  reg.fit <- NULL  
  if (intercept) {
    my.formula <- y ~ x + 1
  } else {
    my.formula <- y ~ x + 0
  }
  ## - OLS in the gaussian case
  if (family == "gaussian") {        
    if (nrow(x) <= ncol(x)) {
      try(reg.fit <- lm.ridge(my.formula, lambda=0), silent=TRUE)
    } else {
      try(reg.fit <- lm(my.formula), silent=TRUE)
    }
  }
  ## - Logistic in the binomial case
  if (family == "binomial") {    
    if (nrow(x) > ncol(x)) {
      try(reg.fit <- glm(my.formula, family="binomial"), silent=TRUE)
    }
  }

  return(reg.fit)
}

sigmoid <- function(eta) {
  return( 1/(1+exp(-eta)) )
}

group.norm.unordered <- function(beta,group) {
  n2 <- c(sqrt(rowsum(beta^2,group,reorder=FALSE)))
  return(n2)
}

group.norm <- function(beta,pk) {
  p <- length(beta)
  K <- length(pk)
  return(c(.C("groupnorm",
              as.integer(p),
              as.integer(K),
              pk, beta, out=as.double(rep(0,K)),
              PACKAGE="scoop"))$out)
}

group.norm.rep <- function(beta,pk) {
  p <- length(beta)
  K <- length(pk)
  return(c(.C("groupnormrep",
              as.integer(p),
              as.integer(K),
              pk, beta, out=as.double(rep(0,p)),
              PACKAGE="scoop"))$out)
}
