\name{scoop}
\alias{scoop}
\alias{scoop.default}
\alias{scoop.formula}
\alias{lasso}
\alias{group.lasso}
\alias{coop.lasso}
\title{Function to fit a penalized regression problem with Lasso,
  group-Lasso or coop-Lasso penalty}
\description{Fit a penalized regression problem either with a Lasso, a group-Lasso or
  a coop-Lasso grouping effect.
  
  \code{lasso}, \code{group.lasso} and \code{coop.lasso} are aliases of
  \code{scoop.default} for which the type of penalty does not need to be specified.  
}
\usage{scoop(x, ...)
\method{scoop}{formula}(formula, data,
            family       = "gaussian",
            penalty      = "coop", 
            intercept    = TRUE,
            normalize    = ifelse(family=="gaussian",TRUE,FALSE),
            lambda       = NULL,
            n.lambda     = 100,
            lambda.min   = 1e-3,
            weights      = NULL,
            verbose      = FALSE,
            eps          = c(1e-5,1e-4),
            max.iter     = NULL,
            max.features = NULL,
            optim.method = ifelse(family=="gaussian","fista","bfgs"),
            ...)

\method{scoop}{default}(x, y, group,
        family       = "gaussian",
        penalty      = "coop", 
        intercept    = TRUE,
        normalize    = ifelse(family=="gaussian",TRUE,FALSE),
        weights      = NULL,
        lambda       = NULL,
        n.lambda     = 100,
        lambda.min   = 1e-3,
        verbose      = FALSE,
        eps          = c(1e-5,1e-4),
        max.iter     = 2*ncol(x),
        max.features = ifelse(missing(group),ncol(x),
                       length(table(group))),
        optim.method = ifelse(family=="gaussian","fista","bfgs"),
        ...)
}

\arguments{
  \item{x}{design matrix (do NOT include intercept).}
  \item{y}{response vector.}
  \item{formula}{formula of the penalized variables. The response
    has to be on the left hand side of \code{~}.}
  \item{data}{data frame containing the variables in the model.}
  \item{group}{vector which defines how
    variables should be grouped. Components sharing the same number define a group.
  }
  \item{family}{response type. For now, either \code{"gaussian"} for linear
    regression or \code{"binomial"} for logistic regression. Default is
    \code{"gaussian"}.}
  \item{penalty}{type of grouping effect in the penalizer. For now, either
    \code{"coop"} (for cooperative-Lasso), \code{"group"} (for
    group-Lasso) or "lasso" (for no grouping effect).}
  \item{intercept}{logical; indicates if an intercept should be included in
    the model. Default is \code{TRUE}.}
  \item{normalize}{logical; indicates if variables should be
    normalized to have unit L2 norm before fitting. Default is
    \code{TRUE} for linear regression, \code{FALSE} for logistic
    regression.}
  \item{lambda}{vector of decreasing penalty levels. If \code{NULL}
    (the default), an appropriate vector will be generated with
    \code{n.lambda}  entries, starting from a level of penalty where only
    the intercept is included, and then shrinked to \code{lambda.min}.
  }
  \item{n.lambda}{integer that indicates the number of values to put
    in the \code{lambda} vector.  Ignored if \code{lambda} is provided.
  }
  \item{lambda.min}{minimal value of penalty that will be
   tried. Default is \code{1e-3}. Ignored if \code{lambda} is provided.} 
  \item{weights}{a vector of real postitive values  that weight the
  penalty of each feature (thus \code{length(weights)=ncol(x)}). If
  \code{NULL} (the default), the penalty of 
  a feature is weighted according to the square root of the size of the
  group it belongs to.}
\item{verbose}{logical; indicates verbose mode to display
  progression. Default is \code{FALSE}.} 
  \item{eps}{tolerances used to stop the optimization algorithm. Default
  is \code{c(1e-5,1e-4)} (see details).}
  \item{max.iter}{maximum number of iterations before the whole optimization
    algorithm stops (for a fixed value of lambda).
  }
  \item{max.features}{integer that indicates the maximum number of
  groups to activate before the algorithm stops. Default is the number
  of level in \code{group} (that is, no early stop in the execution of
  the algorithm).}
  \item{optim.method}{The optimization method used to solve the underlying
  optimization problem on the currently activated set of variables. Can
  be \code{"bfgs"} (a BFGS quasi-Newton method with box constraints),
  \code{"ista"} or   \code{"fista"} (proximal methods). Default is
  \code{"fista"} for linear regression and \code{"bfgs"} for logistic
  regression (the fastest in each category of problem).}
  \item{\dots}{only present for \code{S3} method compatibility.}
}
\details{The general strategy of the algorithm relies on maintaining a
  working set of variable, starting from a vector of zero and entering
  the features by groups of variables. The underlying optimization
  problem is solved only on the activated groups, thus solving small
  smooth problems with increasing size. The working set algorithm
  stops when the optimality conditions are met, up to a tolerance level
  defined by the first element of the \code{eps} argument. The second
  element of this vector controls the convergence of the underlying, smooth
  optimization problem on the currently activated variable. Various
  methods are implemented to solve this latter problem, which is
  controlled by \code{optim.method}. The Lasso is implemented here and
  enjoys the tools of the \pkg{scoop} package, yet note that it is not
  as performant as in\pkg{lars} or \pkg{glmnet}.
}

\value{%
  Returns an object of class \code{scoop} for which
  \code{fitted(object, ...)}, \code{predict(object, newx=NULL, ...)},
  \code{residuals(object, ...)}, \code{print(object, ...)} and
  \code{deviance(object, ...)} generic methods are available. These
  methods are undocumented since the use of the other methods is
  straightforward.  Three documented methods, \code{plot},
  \code{crossval} and \code{selection}, are also available.
  
  A \code{scoop} object is  list-like and contains the following:
  \item{coefficients}{matrix of coefficients with respect to the original input
    (even when normalized). The number of rows of \code{coefficients}
    corresponds the length of \code{lambda}.
  }
  \item{lambda}{vector of penalty levels for which the model has
  eventually been fitted.}
  \item{group}{vector which defines the grouping of the variables.}
  \item{penalty}{type of grouping effect in the penalizer: either
  \code{"coop"} or \code{"group"}. Default is \code{"coop"}.}
  \item{reg.fit}{the corresponding unpenalized fit (either OLS in the
    Gaussian case or logistic in the binomial case). In the Gaussian case,
    if \if{latex}{\eqn{n<p}}\if{html}{n<p}\if{text}{n<p}, OLS is replaced
    with ridge regression with minimal value of regularization. In the
    Binomial case, \code{reg.fit} is \code{NULL} when
    \if{latex}{\eqn{n<p}}\if{html}{n<p}\if{text}{n<p}.}
  \item{it}{The number of iterations in the active set algorithm for
  each value of lambda.}
  \item{it.opim}{the number of iterations for each call to the
    optimization function.}
  \item{dual.gap}{a vector with the same length as lambda which gives
  the duality gap reached for each fit.}
  \item{x}{design matrix saved for further use.}
  \item{y}{response vector saved for further use.}
  \item{call}{call to the \code{scoop} function.}
}
\author{Julien Chiquet}

\seealso{See also \code{\link{plot.scoop}},
  \code{\link{selection.scoop}} and \code{\link{crossval.scoop}}.}
\examples{
\dontrun{
set.seed(87)
## true parameters (including intercept)
beta <- c(2, 0.8, 1.1, 0, -1.5, 0.3, -1.0)
## two groups with three components
group <- c(1, 1, 1, 2, 2, 2)
## random design and additive noise with sd = 2
n <- 60
p <- 6 
x <- matrix(rnorm(p * n), nrow = n)
y <- cbind(1, x) \%*\% beta + rnorm(n,0,2)
## fit coop-lasso and look for BIC selection capability
coolasso <- scoop(x, y, group, lambda.min=0.1)
scoo     <- selection(coolasso)
## plot the coefficient path with the chosen model
plot(coolasso, crit=scoo$BIC)}
}
\keyword{models}
\keyword{regression}
